/* 
 * File:   main.c
 * Author: jrich & mwainwright
 *
 * Created on Jan 21, 2019, 12:42 PM
 * Fixed Jan 15 2020
 * Updated May 15 2021
 * PIC12F1840
 * MPLAB X IDE v5.45
 * XC8 (v2.20)
 */

//Pin IO
//1 = +5
//2 = RA5/CCP1 (PWM) - DDS output
//3 = RA4/AN3 (Phaseshift) - readADC (2)
//4 = MCLR/VPP
//5 = RA2 using external interrupt flag - tactile pre-set
//6 = RA1/AN1/ICSPCLK - readADC (1)
//7 = RA0/ICSPDAT
//8 = GND  

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection->INTOSC oscillator: I/O function on CLKIN pin
#pragma config WDTE = OFF    // Watchdog Timer Enable->WDT disabled
#pragma config PWRTE = OFF    // Power-up Timer Enable->PWRT disabled
#pragma config MCLRE = ON    // MCLR Pin Function Select->MCLR/VPP pin function is MCLR
#pragma config CP = OFF    // Flash Program Memory Code Protection->Program memory code protection is disabled
#pragma config CPD = OFF    // Data Memory Code Protection->Data memory code protection is disabled
#pragma config BOREN = ON    // Brown-out Reset Enable->Brown-out Reset enabled
#pragma config CLKOUTEN = OFF    // Clock Out Enable->CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin
#pragma config IESO = ON    // Internal/External Switchover->Internal/External Switchover mode is enabled
#pragma config FCMEN = ON    // Fail-Safe Clock Monitor Enable->Fail-Safe Clock Monitor is enabled

// CONFIG2
#pragma config WRT = OFF    // Flash Memory Self-Write Protection->Write protection off
#pragma config PLLEN = ON    // PLL Enable->4x PLL enabled
#pragma config STVREN = ON    // Stack Overflow/Underflow Reset Enable->Stack Overflow or Underflow will cause a Reset
#pragma config BORV = LO    // Brown-out Reset Voltage Selection->Brown-out Reset Voltage (Vbor), low trip point selected.
#pragma config LVP = ON    // Low-Voltage Programming Enable->Low-voltage programming enabled


////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <xc.h>

#define LED  PORTAbits.RA0 // RA0 (pin 7)
#define _XTAL_FREQ 32000000
#define ADCStart ADCON0bits.GO_nDONE = 1 //set this bit to begin ADC conversion
//////////////////////////////////////////////////////////////////////////////

unsigned char ctr = 1; // counter for presets
unsigned char ctr2 = 1; // counter2 for other presets


//counter step variables
unsigned char step;
unsigned char step1;
unsigned char step2;
unsigned char step3;


const unsigned char sine[256] = {// sine wave 8 bit resolution scaled to 90% max val
    131, 132, 135, 137, 140, 143, 146, 149, 152, 155, 157, 160, 163, 166, 168, 171,
    174, 176, 179, 181, 184, 186, 189, 191, 194, 196, 198, 200, 202, 205, 207, 209,
    211, 212, 214, 216, 218, 219, 221, 223, 224, 226, 227, 228, 229, 231, 232, 233,
    234, 234, 235, 236, 237, 237, 238, 238, 239, 239, 239, 239, 240, 240, 240, 239,
    239, 239, 239, 238, 238, 237, 237, 236, 236, 235, 234, 233, 232, 231, 230, 229,
    227, 226, 225, 223, 222, 220, 219, 217, 216, 214, 212, 210, 208, 206, 204, 202,
    200, 198, 196, 194, 192, 189, 187, 185, 182, 180, 177, 175, 173, 170, 167, 165,
    162, 160, 157, 154, 152, 149, 146, 144, 141, 138, 135, 133, 130, 127, 124, 122,
    119, 116, 113, 111, 108, 105, 103, 100, 97, 95, 92, 89, 87, 84, 82, 79,
    77, 74, 72, 69, 67, 64, 62, 60, 58, 55, 53, 51, 49, 47, 45, 43,
    41, 39, 37, 36, 34, 32, 31, 29, 28, 26, 25, 24, 22, 21, 20, 19,
    18, 17, 16, 15, 15, 14, 13, 13, 12, 12, 12, 11, 11, 11, 11, 11,
    11, 11, 11, 12, 12, 12, 13, 14, 14, 15, 16, 16, 17, 18, 19, 21,
    22, 23, 24, 26, 27, 29, 30, 32, 33, 35, 37, 39, 41, 43, 45, 47,
    49, 51, 53, 56, 58, 60, 63, 65, 68, 70, 73, 75, 78, 81, 83, 86,
    89, 92, 94, 97, 100, 103, 106, 108, 111, 114, 117, 120, 123, 126, 129, 130
};

const unsigned char myarray[256] = {// myarray 8 bit resolution. 
    //Microcomputer music is always in-flux, noisy and may be overwritten.
    217, 211, 217, 214, 208, 202, 252, 249, 252, 234, 240, 243, 234, 243, 234, 226, 223, 211, 199, 205,
    197, 194, 211, 220, 211, 199, 194, 194, 199, 205, 211, 223, 223, 255, 249, 243, 243, 249, 246,
    243, 243, 234, 214, 197, 199, 214, 197, 208, 197, 191, 199, 191, 156, 104, 168, 217, 205, 199,
    208, 220, 211, 205, 197, 199, 208, 202, 182, 121, 60, 0, 17, 37, 104, 168, 217, 208, 194, 199,
    211, 223, 252, 249, 252, 249, 249, 243, 249, 249, 252, 234, 234, 240, 243, 234, 228, 240, 240,
    234, 231, 223, 252, 249, 249, 252, 252, 249, 243, 240, 237, 231, 228, 240, 243, 249, 243, 249,
    252, 249, 246, 240, 234, 223, 226, 223, 226, 226, 214, 197, 194, 199, 208, 194, 211, 220, 226,
    228, 231, 220, 231, 226, 223, 217, 205, 211, 228, 246, 246, 246, 246, 237, 231, 243, 234, 214,
    223, 234, 231, 234, 214, 223, 255, 249, 246, 246, 249, 246, 237, 240, 240, 231, 243, 249, 252,
    249, 249, 243, 240, 240, 246, 243, 249, 249, 249, 252, 249, 246, 249, 252, 252, 234, 243, 249,
    249, 252, 249, 243, 249, 249, 249, 246, 237, 226, 223, 234, 240, 246, 240, 237, 231, 226, 228,
    226, 234, 243, 249, 243, 234, 214, 223, 165, 104, 168, 231, 237, 226, 182, 136, 78, 20, 37, 104,
    168, 231, 237, 226, 214, 208, 194, 199, 202, 208, 217, 217, 199, 202, 182, 121, 60, 0, 17, 37,
    104, 168, 234, 234, 228
};

////////////////////////////////Global variable here//////////////////////////////////////////////
long PhaseAccum; //phase accumulator generates the cycle rate for lookup table
//loading thereby changing frequency. The MSbyte is used to provide the byte address
//of the look up table value to be used.
long PhaseShift; //value added to PhaseAccum every PWM cycle. This makes the waveform
//lookup faster or slower which changes frequency.
////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////PIC Config routine here////////////////////////////////////

//map function // eg map(readADC(1), 0, 1023, 0, 255);
//long map(long x, long in_min, long in_max, long out_min, long out_max) {
//    return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
//}

void Init_Main() {
    //PIC12F1840 specific config

    OPTION_REG = 0b11000000; // disable internal pull ups
    OSCCON = 0b11110000; // 8MHz clk //32Mhz pll
    TRISA = 0b00011111; // configure IO/A2D(GPIO0) and MCLR set as inputs // direction in/out
    T2CON = 0b00000100; // TMR2 ON, postscale 1:1, prescale 1:1
    PR2 = (0x50); // sets PWM rate to approx 98.5KHz with 32Mhz internal oscillator
    CCP1CON = 0b00001111; // CCP1 ON, and set to simple PWM mode (pin 2)
    APFCONbits.CCP1SEL = 1; // RA2 = 0 RA5 = 1 //alternative pin function //tactile select
    PhaseShift = 0x00FFFFFF; // frequency values loaded into //0x00FFFFFF = 16777215 (24 bit))
    ANSELA = 0b00010010; // select RA4 (pin 3) and RA1 (pin 6) as A2D inputs // ANSELA select as analogue or digital // PhaseShift and Touch
    //orig
    ADCON0 = 0b00001101; // configure ADC
    ADCON1 = 0b00100000; // configure ADC //  1 = right justification (bit 7) // 0 = left just. Seems to produce a 16-bit value!

    INTCON = 0b10010010; // config interrupt - GIE and INTE // INTCONbits.INTF = 1; void __interrupt
}



// interrupt service routine - pre-set select tactile switch random!

// void interrupt ISR(void) {
   void __interrupt () ISR(void) {
    if (INTCONbits.INTF == 1) { //using external interrupt flag @ RA2
    //  __delay_ms(1); // debounce
    //    if (INTCONbits.INTF == 1){ // if still pressed      
            ctr = (rand () % 8) + 1; // increment the counter 1-9
            ctr2 = (rand () % 4) + 1; // increment the counter2 1-5
            INTCONbits.INTF = 0; //clear flag
       // }
//        //number of presets here!
//        if (ctr == (9 + 1)) // if counter reaches n, + 1 more than presets,
//            ctr = 1; // reset counter (min no.)
//        if (ctr2 == (5 + 1)) // if counter reaches n, + 1 more than presets,
//            ctr2 = 1; // reset counter (min no.)
    }
}

unsigned int readADC(unsigned char channel) {
    unsigned int AN_Val;

    // Select desired channel for ADC operation
    switch (channel) {
        case 1: // AN1 (pin 6) touch
            ADCON0bits.CHS = 1;
            break;
        case 2: // AN3 (pin 3) PhaseShift
            ADCON0bits.CHS = 3;
            break;
        default: // Any other value will default to:
            ADCON0bits.CHS = 0; // 
    }
    // Channel selected proceed with ADC conversion

    __delay_us(5); // sampling time // can be c. 5 us - acquisition time
    // start a conversion
    ADCON0bits.GO = 1; //This sets the go/!done bit that starts conversion. Bit will be cleared when ADC is complete

    while (ADCON0bits.GO == 1)
        //    __delay_us(10); // longer delay here influences the ISR and timers // can afford to leave this out

        AN_Val = ((ADRESH << 8) + ADRESL); // returns 10-bit // NOTE justification ADCON1!
    //AN_Val = ADRESH; //returns 8-bit

    return AN_Val; // int ADC(unsigned char channel)
}


////////////////////////////main program loop here/////////////////////////////////

void main() {
    Init_Main(); //config part  

    unsigned int AN_Val;

    while (1) { //alway do this  
        //__delay_us(5); //testing the sample time for readADC function. Lower pitch and more aliasing.
        // written in nested loop below less aliasing

        //        ////ADC channel 1 - LED test//////
        //
        //                if (readADC(1) >= 32000) // NOTE justification ADCON1 needs to be right just for 10-bit? // left just @ 16-bit (65535)? 
        //                LED = 1;
        //                else
        //                LED = 0;
        //                
        //        /////////////

        while (!PIR1bits.TMR2IF); // wait for TMR2 cycle to restart //  nested loop?

        // Resolution of PWM. Standard up to 10 bits. Here set at 8 bit? (6 bits in MSbits - CCPRIL/2 bits in CCP1CON)
        // CCP1 RA5 = pin 2 (output)
        // What happens when bitwise operator is altered = PWM resolution? >> 4 quieter and lower freq. No bitwise? Distorted. Changes waveshape ramp-like
        // Moved to the left get PWM lower res and a kind of bit crushing  << 4
        // &PhaseAccum)[3] 3? 
        // (char *) is a pointer to character at an address 
        // (char *) & = a cast? convert type int - char?


        //some step variables
        step ^= ADRESL;
        //step1 not used
        step2 += 1;
        step3 += 3;
        
        ////// 5 different waveform generators 'cores' ///////

        if (ctr2 == 1) { //nosy
           CCPR1L = (sine[((char *) &PhaseAccum)[3]])+step +1;// * 2; // load MSbits 7-2 duty cycle value into CCPRIL
           CCP1CON ^= ((sine[step]) & 0x03) <<4; // load in bits 1-0 into 5 and 4 of CCP1CON
         //   step ^= ADRESL;
        }

        if (ctr2 == 2) { //standard bog
            CCPR1L = (sine[((char *) &PhaseAccum)[3]]) >> 2; // load MSbits 7-2 duty cycle value into CCPRIL
            CCP1CON ^= ((sine[((char *) &PhaseAccum)[3]]) & 0x03) << 4; // load in bits 1-0 into 5 and 4 of CCP1CON
        }

        if (ctr2 == 3) { //sawww
            CCPR1L = ((char *) &PhaseAccum)[3] >> 2; // load MSbits 7-2 duty cycle value into CCPRIL               
            CCP1CON ^= ((char *) &PhaseAccum)[3] & 0x03 << 4; // load in bits 1-0 into 5 and 4 of CCP1CON // 0x03 = 3
        }

        if (ctr2 == 4) { //vinegar
            CCPR1L = (sine[ ((char *) &PhaseAccum) [3] & readADC(1)]) >> 2; // load MSbits 7-2 duty cycle value into CCPRIL              
            CCP1CON ^= ((sine[((char *) &PhaseAccum)[3]]) & 0x03) << 4; // load in bits 1-0 into 5 and 4 of CCP1CON // 0x03 = 3 
        }

        if (ctr2 == 5) { //myarray
            CCPR1L = (myarray[((char *) &PhaseAccum)[3]]) >> 2; // load MSbits 7-2 duty cycle value into CCPRIL
            CCP1CON ^= ((myarray[((char *) &PhaseAccum)[3]]) & 0x03) << 4; // load in bits 1-0 into 5 and 4 of CCP1CON
        }


        ////duty cycle value byte is now loaded for next cycle coming//////

        if (PIR1bits.ADIF == 0)
            ADCStart; //start ADC here to get value into PhaseShift to change Freq

        //will have to go around the loop one time before ready flag is high

        if (PIR1bits.ADIF) //if ADC read complete load values and clear flag 
            // readADC(2) here seems to change the channel, also loads ADC value into ADRESL and ADRESH.
            // although is this then duplicated or overwritten below - ((char *) &PhaseShift)[1] = ADRESL etc.?
            readADC(2);

        ((char *) &PhaseShift)[1] = ADRESL; //load ADRESL into PhaseShift
        ((char *) &PhaseShift)[2] = ADRESH; //load ADRESH into PhaseShift 


        PIR1bits.ADIF = 0; //clear flag so next time ADC can?run        
        PIR1bits.TMR2IF = 0; // clear TMR2 int flag 


        ////// 9 algorithms //////
        
        //stuff
        if (ctr == 1){
            PhaseAccum = PhaseAccum + ((PhaseShift * sine[readADC(1)]) + 1);
         }
        //touch me
        if (ctr == 2){
            PhaseAccum = PhaseAccum + ((PhaseShift << 6) * readADC(1) + 1);
        }
        //whistle FM
        if (ctr == 3){
            PhaseAccum = PhaseAccum + (((PhaseShift << readADC(1)) * sine[step2]) + 1);
        //step2 += 1; //see variables
        }
        //brocken
        if (ctr == 4){
            PhaseAccum = ((PhaseShift * sine[ADRESL]) + PhaseShift + readADC(1));
        }
        //starling
        if (ctr == 5){
            PhaseAccum = PhaseAccum % (PhaseAccum + step3) + (((PhaseShift * sine[step3 >> readADC(1)])));
        //step3 += 3;
        }
        //sine
        if (ctr == 6){// brackets added!
            PhaseAccum = PhaseAccum + ((PhaseShift << 6) + 1);
        }
        //brownouts
        if (ctr == 7){
            PhaseAccum = ((PhaseShift * sine[ADRESL]) + PhaseShift)*2232;
        }
        //speed-dial
        if (ctr == 8){
            PhaseAccum = PhaseAccum + (PhaseShift << (myarray[readADC(1)] >> 5));
        PhaseShift ^= readADC(1);
        }
        //strange memory effect
        if (ctr == 9){
            ADRESL = readADC(1)*PhaseShift;  
        PhaseAccum = PhaseAccum + ((PhaseShift * (sine[readADC(1)*6] )));
        }
    }
}

